import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * This counter does three main things.
 * It increments your page counter, saves some statistical
 * data from the client and additionally prints out the date
 * the file was last modified and since what time the counter
 * is in use.
 * <p>Here is an example how to use it for a html page:<br>
 * <pre>
 * &lt;servlet name=cclcounter code=CCLCounterServlet&gt;
 * &lt;param name=timezone value=CET&gt;
 * &lt;param name=systemtimezone value=UTC&gt;
 * &lt;param name=offset value=245&gt;
 * &lt;param name=startdate value="Feb 27 1996"&gt;
 * &lt;param name=ip1 value=129.26.121.4&gt;
 * &lt;param name=ip2 value=129.26.40.42&gt;
 * &lt;param name=ip3 value=129.26.13.252&gt;
 * &lt;/servlet&gt;
 * </pre><br>
 *
 * @version $Id: CCLCounterServlet.java,v 1.1 1997/04/09 23:25:08 lahme Exp lahme $
 * @author  Chr. Clemens Lahme
 */
public class CCLCounterServlet extends HttpServlet {
	public String getServletInfo() {
		return "CCLCounter Version 1.1 4/3/1997 by Chr. Clemens Lahme";
	}

	private static final int HOUR = 60*60*1000;

	private static final String S_PARAM_TIME_ZONE_ID = "timezone";
	private static final String S_PARAM_SYSTEM_TIME_ZONE_ID = "systemtimezone";
	private static final String S_PARAM_COUNTER_OFFSET = "offset";
	private static final String S_PARAM_START_DATE = "startdate";

	private static final String S_COUNT_POSTFIX = ".count";

	/**
	 * Used timezone ID.
	 * <br>Can be modified using the servlet parameter: timezone=GMT
	 * <br>Default: CET
	 * @see TimeZone#getAvailableIDs
	 */
	protected String _sTimeZoneID;

	/**
	 * CCLCounter uses the non portable function File.lastModified.<br>
	 * Under Windows 95 this gives back the time in UTC.
	 * Under Solaris the local timezone is used.
	 * <br>Can be modified using the servlet parameter: systemtimezone=UTC
	 * <br>For a server on Windows 95 the value UTC should be used instead of local time.
	 * <br>Default value: the same as _sTimeZoneID
	 * @see File#lastModified
	 * @see _sTimeZoneID
	 */
	protected String _sSystemTimeZoneID = null;

	private String _sCountingStartDate = null;

	/**
	 * Can be modified using the servlet parameter: counteroffset=200
	 */
	protected long _lCounterOffset = 0;

	private static String[] _asMonthName = {"Jan", "Feb", "Mar",
														 "Apr", "May", "Jun",
														 "Jul", "Aug", "Sep",
														 "Oct", "Nov", "Dec"};

	private static Hashtable _htHtmlPages = new Hashtable();

	/**
	 * Remote Address is not fast but is needed in two different
	 * functions. Therefore it is saved in a global variable.
	 * @see _processParameter(ServletRequest)
	 */
	private String _sRemAddress = null;

	private boolean _isClientPageAdmin(ServletRequest pServletRequest_)
	{
		for(Enumeration e = pServletRequest_.getParameterNames();
			 e.hasMoreElements(); )
		{
			String sName = (String)e.nextElement();
			if (_sRemAddress.equals(pServletRequest_.
											getParameter(sName)))
			{
				return true;
			}
		}

		return false;
	}

	protected void _processParameter(ServletRequest pServletRequest_) {
		if (pServletRequest_.getParameter(S_PARAM_TIME_ZONE_ID) !=
			 null)
		{
			_sTimeZoneID = (String)pServletRequest_.
					 getParameter(S_PARAM_TIME_ZONE_ID);
		} else {
			_sTimeZoneID = "CET";
		}
		if (pServletRequest_.getParameter(S_PARAM_SYSTEM_TIME_ZONE_ID) !=
			 null)
		{
			_sSystemTimeZoneID = (String)pServletRequest_.
					 getParameter(S_PARAM_SYSTEM_TIME_ZONE_ID);
		} else {
			_sSystemTimeZoneID = new String(_sTimeZoneID);
		}
		if (pServletRequest_.getParameter(S_PARAM_START_DATE) !=
			 null)
		{
			_sCountingStartDate = (String)pServletRequest_.
					 getParameter(S_PARAM_START_DATE);
		} else {
			_sCountingStartDate = null;
		}
		if (pServletRequest_.getParameter(S_PARAM_COUNTER_OFFSET) !=
			 null)
		{
			_lCounterOffset = (Long.valueOf
					 ((String)pServletRequest_.getParameter
					  (S_PARAM_COUNTER_OFFSET))).longValue();
		} else {
			_lCounterOffset = 0;
		}

		// Remote Address is also not fast and is needed at different
		//    occasions and therefore is saved in a global variable.
		// Remote Host is too slow
		_sRemAddress = pServletRequest_.getRemoteAddr();
		if (_sRemAddress == null) {
			_sRemAddress = "";
		}
	}

	protected Calendar _getCalendar(String sTimeZoneID_) {
		TimeZone pTimeZone = null;
		if (sTimeZoneID_.equals("UTC")) {
			pTimeZone = new SimpleTimeZone(0, "UTC");
		} else {
			pTimeZone = SimpleTimeZone.getTimeZone(sTimeZoneID_);
		}
		// New Daylight Saving Time Convention in 35 European Countries
		if (sTimeZoneID_.equals("CET")) {
			pTimeZone = new SimpleTimeZone(1*HOUR, "CET",
													 Calendar.MARCH, -1,
													 Calendar.SUNDAY,
													 2*HOUR,
													 Calendar.OCTOBER, -1,
													 Calendar.SUNDAY,
													 2*HOUR);
		}
		Calendar pCalendar = new GregorianCalendar(pTimeZone);

		return pCalendar;
	}

	protected String _getLastModified(String sHtmlFullFileName_) {
		String sLastModified = "";

		File flHtmlPage = new File(sHtmlFullFileName_);
		Calendar pCalendar = _getCalendar(_sSystemTimeZoneID);
		if (flHtmlPage != null) {
			Date dtLastModified = new Date(flHtmlPage.lastModified());
			pCalendar.setTime(dtLastModified);
			sLastModified += _asMonthName[pCalendar.get(Calendar.MONTH)] + " " +
					 pCalendar.get(Calendar.DAY_OF_MONTH) + " " +
					 pCalendar.get(Calendar.YEAR) + "&nbsp;";
			int hours = pCalendar.get(Calendar.HOUR_OF_DAY);
			if (hours < 10) {
				sLastModified += "0";
			}
			sLastModified += hours + ":";
			int minutes = pCalendar.get(Calendar.MINUTE);
			if (minutes < 10) {
				sLastModified += "0";
			}
			sLastModified += minutes + ":";
			int seconds = pCalendar.get(Calendar.SECOND);
			if (seconds < 10) {
				sLastModified += "0";
			}
			sLastModified += seconds;
			sLastModified = "Last modified: " + sLastModified +
					 " " + _sTimeZoneID;
		}

		return sLastModified;
	}

	protected Object _getPageLock(String sPage_) {
		if (!_htHtmlPages.contains(sPage_)) {
			_htHtmlPages.put(sPage_, new Object());
		}

		return _htHtmlPages.get(sPage_);
	}

	protected String _formatCreation(Date dtCreation_) {
		if (_sCountingStartDate != null) {
			return new String(_sCountingStartDate);
		}

		Calendar pCalendar = _getCalendar(_sTimeZoneID);
		pCalendar.setTime(dtCreation_);
		String sCreation = _asMonthName[pCalendar.get(Calendar.MONTH)] + " " +
				 pCalendar.get(Calendar.DAY_OF_MONTH) + " " +
				 pCalendar.get(Calendar.YEAR);

		return sCreation;
	}

	protected void _print(ServletOutputStream pServletOutputStream_,
								 long lCounter_, String sCreation_,
								 String sLastModified_)
			 throws IOException
	{
		pServletOutputStream_.println("This page has been accessed " +
												lCounter_  + " times since " +
												sCreation_ + ". " +
												sLastModified_ + "<br>");
		// Just so I can find all the pages in the web that use
		// this counter ;)
		pServletOutputStream_.println("<!--" + getServletInfo() + "-->");
	}

	protected void _save(String sHtmlFullFileName_, Date dtCreation_,
								long lCounter_)
	{
		try {
			FileOutputStream pFileOutputStream =
					 new FileOutputStream(sHtmlFullFileName_ +
												 S_COUNT_POSTFIX);
			ObjectOutput pObjectOutput = new ObjectOutputStream(pFileOutputStream);
			pObjectOutput.writeObject(dtCreation_);
			pObjectOutput.writeObject(new Long(lCounter_));
			pObjectOutput.flush();
			pObjectOutput.close();
		} catch(Exception e) {
		}
	}

	protected void _snoop(String sHtmlFullFileName_,
								 long lCounter_,
								 HttpServletRequest pHttpServletRequest_)
	{
		String sSnoopFullFileName = sHtmlFullFileName_ + ".snoop";
		try {
			RandomAccessFile raflSnoop = new RandomAccessFile(sSnoopFullFileName, "rw");
			raflSnoop.seek(raflSnoop.length());

			String sCounter = "" + lCounter_ + "\n";
			String sDate = "" + new Date() + "\n";

			String sRemAddress = _sRemAddress + "\n";
			// Referer
			String sReferer = pHttpServletRequest_.getHeader("Referer");
			if (sReferer == null) {
				sReferer = "";
			}
			sReferer += "\n";
			// Browser
			String sBrowser = pHttpServletRequest_.getHeader("User-Agent");
			if (sBrowser == null) {
				sBrowser = "";
			}
			sBrowser += "\n";

			raflSnoop.writeBytes(sCounter + sDate + sRemAddress +
										sReferer + sBrowser);
			raflSnoop.close();
		} catch(Exception pException) {
		}
	}

	public CCLCounterServlet() {
		super();
	}

	public void service(HttpServletRequest pHttpServletRequest_,
							  HttpServletResponse pHttpServletResponse_)
			 throws ServletException, IOException
	{
		pHttpServletResponse_.setContentType("text/html");

		_processParameter(pHttpServletRequest_);

		String sHtmlFullFileName = pHttpServletRequest_.
				 getPathTranslated();
		String sLastModified = _getLastModified(sHtmlFullFileName);

		synchronized(_getPageLock(sHtmlFullFileName)) {
			// load
			Date dtCreation = new Date();
			long lCounter = _lCounterOffset;
			try {
				FileInputStream pFileInputStream =
						 new FileInputStream(sHtmlFullFileName +
													S_COUNT_POSTFIX);
				ObjectInputStream oisCount = new ObjectInputStream(pFileInputStream);
				dtCreation = (Date)oisCount.readObject();
				lCounter += ((Long)oisCount.readObject()).longValue();
				oisCount.close();
			} catch(Exception e) {
			}
			String sCreation = _formatCreation(dtCreation);

			// Increment
			boolean bIncrement = !_isClientPageAdmin(pHttpServletRequest_);
			if (bIncrement) {
				lCounter++;
			}

			// Print
			_print(pHttpServletResponse_.getOutputStream(),
					 lCounter, sCreation, sLastModified);

			if (!bIncrement) {
				return;
			}


			// save
			_save(sHtmlFullFileName, dtCreation, lCounter - _lCounterOffset);

			// snoop
			_snoop(sHtmlFullFileName, lCounter, pHttpServletRequest_);
		}
	}
}
